<?php

namespace Kahanit\TableRateShipping\Helper;

/**
 * Class TRSHelper
 *
 * @package Kahanit\TableRateShipping\Helper
 */
class TRSHelper
{
    public static function addslashes($str, $enclosed = false)
    {
        if (is_array($str)) {
            foreach ($str as &$item) {
                $item = TRSHelper::addslashes($item, $enclosed);
            }

            return $str;
        }

        $search = ["\\", "\0", "\n", "\r", "\x1a", "'", '"'];
        $replace = ["\\\\", "\\0", "\\n", "\\r", "\Z", "\'", '\"'];

        if ($enclosed) {
            $str = TRSHelper::addslashes($str);
        }

        return str_replace($search, $replace, $str);
    }

    public static function stripslashes($str)
    {
        if (is_array($str)) {
            foreach ($str as &$item) {
                $item = TRSHelper::stripslashes($item);
            }

            return $str;
        }

        $search = ["\\\\", "\\0", "\\n", "\\r", "\Z", "\'", '\"'];
        $replace = ["\\", "\0", "\n", "\r", "\x1a", "'", '"'];

        return str_replace($search, $replace, $str);
    }

    public static function filterSearchQuery($search_query = '', $keys = [])
    {
        $search_query = trim($search_query);
        $search_items = [];

        foreach ($keys as $key) {
            $pattern = '/' . $key . '\s*:\s*[^:]*((?=' . implode('\s*:)|(?=', $keys) . '\s*:)|($))/i';
            preg_match($pattern, $search_query, $matches);

            if (isset($matches[0])) {
                $search_value = preg_replace('/\s+/', ' ', $matches[0]);
            } else {
                $search_value = '';
            }

            $search_value = explode(':', $search_value);
            $search_items[$key] = end($search_value);
            $search_items[$key] = trim($search_items[$key]);
        }

        $search_items['query'] = $search_query;

        return $search_items;
    }

    public static function checkZipCode($zip_code, $zip_code_format, $iso_code)
    {
        $iso_code_wildcard = preg_replace('/(.)/i', '([$1_%*]|)', $iso_code);

        $zip_regexp = '/^' . $zip_code_format . '$/ui';
        $zip_regexp = str_replace(' ', '( |)', $zip_regexp);
        $zip_regexp = str_replace('-', '(-|)', $zip_regexp);
        $zip_regexp = str_replace('N', '([0-9_%*]|)', $zip_regexp);
        $zip_regexp = str_replace('L', '([a-zA-Z_%*]|)', $zip_regexp);
        $zip_regexp = str_replace('C', $iso_code_wildcard, $zip_regexp);

        return (bool)preg_match($zip_regexp, $zip_code);
    }
}
