<?php

namespace Kahanit\TableRateShipping\Model\Carrier;

// inheritance
use Magento\Shipping\Model\Carrier\AbstractCarrier;
use Magento\Shipping\Model\Carrier\CarrierInterface;
// injection
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Quote\Model\Quote\Address\RateResult\ErrorFactory;
use Psr\Log\LoggerInterface;
use Magento\Shipping\Model\Rate\ResultFactory;
use Magento\Quote\Model\Quote\Address\RateResult\MethodFactory;
use Kahanit\TableRateShipping\Model\CarrierTableRate;
use Kahanit\TableRateShipping\Model\CarrierTableRateMethod;
use Magento\Catalog\Model\ProductFactory;
use Magento\Checkout\Model\Session as CheckoutSession;
// other
use Magento\Quote\Model\Quote;
use Magento\Quote\Model\Quote\Item as QuoteItem;
use Magento\Quote\Model\Quote\Address\RateRequest;
use Magento\Catalog\Model\Product\Type as ProductType;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable as ConfigurableProductType;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Directory\Model\CurrencyFactory;

class TableRateShipping extends AbstractCarrier implements CarrierInterface
{
    protected $_code = 'tablerateshipping';

    protected $rateResultFactory;

    protected $rateMethodFactory;

    protected $carrierTableRate;

    protected $carrierTableRateMethod;

    protected $productFactory;

    protected $checkoutSession;

    protected $products;

    protected $carriers;

    protected $storeManager;

    protected $currencyFactory;

    public function __construct(
        ScopeConfigInterface $scopeConfig,
        ErrorFactory $rateErrorFactory,
        LoggerInterface $logger,
        ResultFactory $rateResultFactory,
        MethodFactory $rateMethodFactory,
        CarrierTableRate $carrierTableRate,
        CarrierTableRateMethod $carrierTableRateMethod,
        ProductFactory $productFactory,
        CheckoutSession $checkoutSession,
        StoreManagerInterface $storeManager,
        CurrencyFactory $currencyFactory,
        array $data = []
    ) {
        $this->rateResultFactory = $rateResultFactory;
        $this->rateMethodFactory = $rateMethodFactory;
        $this->carrierTableRate = $carrierTableRate;
        $this->carrierTableRateMethod = $carrierTableRateMethod;
        $this->productFactory = $productFactory;
        $this->checkoutSession = $checkoutSession;
        $this->storeManager = $storeManager;
        $this->currencyFactory = $currencyFactory;

        parent::__construct($scopeConfig, $rateErrorFactory, $logger, $data);
    }

    public function getAllowedMethods()
    {
        return [$this->getCarrierCode() => __($this->getConfigData('name'))];
    }

    public function collectRates(RateRequest $request)
    {
        if (!$this->isActive()) {
            return false;
        }

        $quote = $this->checkoutSession->getQuote();
        $storeId = $quote->getStoreId();

        // product details
        $productDetails = $this->getProductDetails($quote, $storeId);

        // carrier details
        $carrierDetails = $this->getCarrierDetails($productDetails, $storeId);

        // prepare rate result
        $shippingAddress = $quote->getShippingAddress();
        $result = $this->rateResultFactory->create();
        foreach ($carrierDetails as $carrierDetailKey => $carrierDetail) {
            $subCarriers = explode('-', $carrierDetailKey);
            $vars = [];
            foreach ($subCarriers as $subCarrier) {
                $vars[$subCarrier] = [
                    'id_carrier' => $this->carriers[$subCarrier]['id_carrier'],
                    'carrier_name' => $this->carriers[$subCarrier]['carrier_name'],
                    'method_name' => $this->carriers[$subCarrier]['method_name'],
                    'tw' => 0,
                    'tp' => 0,
                    'tptp' => 0,
                    'tq' => 0,
                    'tv' => 0,
                    'skip' => -1,
                    'stop' => false
                ];
                $subCarrierProducts = $carrierDetail['y-' . $subCarrier];
                foreach ($subCarrierProducts as $subCarrierProduct) {
                    $vars[$subCarrier]['tw'] = $vars[$subCarrier]['tw'] + $productDetails[$subCarrierProduct]['weight'];
                    $vars[$subCarrier]['tp'] = $vars[$subCarrier]['tp'] + $productDetails[$subCarrierProduct]['price'];
                    $vars[$subCarrier]['tptp'] = $vars[$subCarrier]['tptp'] + $productDetails[$subCarrierProduct]['ptprice'];
                    $vars[$subCarrier]['tq'] = $vars[$subCarrier]['tq'] + $productDetails[$subCarrierProduct]['qty'];
                    $vars[$subCarrier]['tv'] = $vars[$subCarrier]['tv'] + $productDetails[$subCarrierProduct]['volume'];
                }
                $vars[$subCarrier]['ts'] = $this->carrierTableRate->getResource()->getShippingPrice(
                    $subCarrier,
                    $shippingAddress->getCountryId(),
                    $shippingAddress->getRegionId(),
                    $shippingAddress->getCity(),
                    $shippingAddress->getPostcode(),
                    $vars[$subCarrier],
                    $storeId
                );
            }

            $shippingPrice = array_column($vars, 'ts');
            foreach ($shippingPrice as $shippingPriceItem) {
                if ($shippingPriceItem === false) {
                    continue 2;
                }
            }
            $carrierName = implode(', ', array_column($vars, 'carrier_name'));
            $methodName = implode(', ', array_column($vars, 'method_name'));
            $carrierId = 'trs-' . implode('-', array_column($vars, 'id_carrier'));
            $shippingPrice = array_sum($shippingPrice);

            if ($shippingPrice !== false) {
                $rateResultMethod = $this->rateMethodFactory->create();
                $rateResultMethod->setCarrier($this->getCarrierCode());
                $rateResultMethod->setCarrierTitle($carrierName);
                $rateResultMethod->setMethodTitle($methodName);
                $rateResultMethod->setMethod($carrierId);
                $rateResultMethod->setPrice($shippingPrice);
                $rateResultMethod->setCost($shippingPrice);
                $result->append($rateResultMethod);
            }
        }

        return $result;
    }

    private function getProductDetails(Quote $quote, $storeId)
    {
        // get all products ids
        $productIds = [];
        $cartItems = $quote->getAllItems();
        foreach ($cartItems as $cartItem) {
            $itemProductId = $cartItem->getProduct()->getId();
            if (!in_array($itemProductId, $productIds)) {
                $productIds[] = $itemProductId;
            }
        }

        // get products details
        $this->products = $this->productFactory->create()->getCollection()
            ->addFieldToSelect('weight')
            ->addFieldToSelect('price')
            ->addAttributeToSelect('volume_type')
            ->addAttributeToSelect('volume')
            ->joinTable(
                ['ctp' => $this->carrierTableRate->getResource()->getTable('carrier_table_rate_product')],
                'id_product = entity_id',
                ['GROUP_CONCAT(id_carrier SEPARATOR \',\') AS id_carriers'],
                ['id_shop' => [$storeId, Store::DEFAULT_STORE_ID]],
                'left'
            )
            ->addFieldToFilter('entity_id', ['IN' => $productIds])
            ->groupByAttribute('entity_id')
            ->getItems();
        $productsTemp = [];
        foreach ($this->products as $product) {
            $productsTemp[$product->getData('entity_id')] = $product->getData();
            if (isset($productsTemp[$product->getData('entity_id')]['volume_type'])) {
                $productsTemp[$product->getData('entity_id')]['volume_type']
                    = $product->getAttributeText('volume_type');
            }
        }
        $this->products = $productsTemp;
        unset($productsTemp);

        // prepare product details
        $productDetails = [];
        $cartItems = $quote->getAllVisibleItems();
        foreach ($cartItems as $cartItemKey => $cartItem) {
            $itemProductId = $cartItem->getProduct()->getId();
            $itemChildren = $cartItem->getChildren();
            $itemProductType = $cartItem->getProductType();
            $productDetailsKey = $cartItemKey . '-' . $itemProductId;

            if (count($itemChildren) == 0) {
                $productDetails[$productDetailsKey] = $this->getSimpleProductDetails($cartItem);
            } else {
                if ($itemProductType == ProductType::TYPE_BUNDLE) {
                    $productDetails[$productDetailsKey] = $this->getConfigurableProductDetails($cartItem);
                } else {
                    if ($itemProductType == ConfigurableProductType::TYPE_CODE) {
                        $productDetails[$productDetailsKey] = $this->getBundleProductDetails($cartItem);
                    } else {
                        $productDetails[$productDetailsKey] = $this->getSimpleProductDetails($cartItem);
                    }
                }
            }

            $store = $this->storeManager->getStore();
            $currencyCurrent = $store->getCurrentCurrency()->getCode();
            $currencyBase = $store->getBaseCurrency()->getCode();

            if (strcasecmp($currencyCurrent, $currencyBase) == 0) {
                $price = $cartItem->getRowTotalInclTax();
                $ptprice = $cartItem->getRowTotal();
            } else {
                $currencyFrom = $this->currencyFactory->create()->load($currencyCurrent);
                $price = $currencyFrom->convert($cartItem->getRowTotalInclTax(), $currencyBase);
                $ptprice = $currencyFrom->convert($cartItem->getRowTotal(), $currencyBase);
            }

            $productDetails[$productDetailsKey]['id_carriers'] = $this->products[$productDetails[$productDetailsKey]['entity_id']]['id_carriers'];
            $productDetails[$productDetailsKey]['price'] = $price;
            $productDetails[$productDetailsKey]['ptprice'] = $ptprice;
            $productDetails[$productDetailsKey]['qty'] = $cartItem->getQty();
        }

        return $productDetails;
    }

    private function getCarrierDetails($productDetails, $storeId)
    {
        $this->carriers = $this->carrierTableRateMethod->getResource()->getCarriers($storeId, true);
        $carriersTemp = [];
        foreach ($this->carriers as $carrier) {
            $carriersTemp[$carrier['id_carrier']] = $carrier;
        }
        $this->carriers = $carriersTemp;
        unset($carriersTemp);

        return $this->getCarriersForProducts($productDetails, $this->carriers);
    }

    private function getSimpleProductDetails(QuoteItem $cartItem)
    {
        $itemProductId = $cartItem->getProduct()->getId();
        $itemProductWeight = $cartItem->getWeight();
        $itemProductVolume = (isset($this->products[$itemProductId]['volume'])) ? $this->products[$itemProductId]['volume'] : 0;
        $itemProductQty = $cartItem->getQty();

        return [
            'entity_id' => $itemProductId,
            'weight' => $itemProductWeight * $itemProductQty,
            'volume' => $itemProductVolume * $itemProductQty
        ];
    }

    private function getConfigurableProductDetails(QuoteItem $cartItem)
    {
        $itemProductQty = $cartItem->getQty();
        $itemChildren = $cartItem->getChildren();
        $itemChild = array_shift($itemChildren);
        $itemChildProductId = $itemChild->getProduct()->getId();
        $itemChildWeight = $itemChild->getWeight();
        $itemChildVolume = (isset($this->products[$itemChildProductId]['volume'])) ? $this->products[$itemChildProductId]['volume'] : 0;

        return [
            'entity_id' => $itemChildProductId,
            'weight' => $itemChildWeight * $itemProductQty,
            'volume' => $itemChildVolume * $itemProductQty
        ];
    }

    private function getBundleProductDetails(QuoteItem $cartItem)
    {
        $itemProductId = $cartItem->getProduct()->getId();
        $itemProductQty = $cartItem->getQty();

        return [
            'entity_id' => $itemProductId,
            'weight' => $this->getBundleProductWeight($cartItem) * $itemProductQty,
            'volume' => $this->getBundleProductVolume($cartItem) * $itemProductQty
        ];
    }

    private function getBundleProductWeight(QuoteItem $cartItem)
    {
        $itemProductWeight = $cartItem->getWeight();
        $itemProductWeightType = $cartItem->getProduct()->getWeightType();
        $itemChildren = $cartItem->getChildren();

        $weight = 0;
        if ($itemProductWeightType) {
            $weight = $itemProductWeight;
        } else {
            foreach ($itemChildren as $itemChild) {
                $weight += $itemChild->getWeight() * $itemChild->getQty();
            }
        }

        return $weight;
    }

    private function getBundleProductVolume(QuoteItem $cartItem)
    {
        $itemProductId = $cartItem->getProduct()->getId();
        $itemProductVolume = (isset($this->products[$itemProductId]['volume'])) ? $this->products[$itemProductId]['volume'] : 0;
        $itemProductVolumeType = (isset($this->products[$itemProductId]['volume_type'])) ? $this->products[$itemProductId]['volume_type'] : 'Fixed';
        $itemChildren = $cartItem->getChildren();

        $volume = 0;
        if (strcasecmp($itemProductVolumeType, 'Fixed') == 0) {
            $volume = $itemProductVolume;
        } else {
            foreach ($itemChildren as $itemChild) {
                $itemChildProductId = $itemChild->getProduct()->getId();
                $itemChildVolume = (isset($this->products[$itemChildProductId]['volume'])) ? $this->products[$itemChildProductId]['volume'] : 0;
                $volume += $itemChildVolume * $itemChild->getQty();
            }
        }

        return $volume;
    }

    private function getCarriersForProducts($products, $carriers)
    {
        $carrierDetails = [];
        $shipsAllCarriers = [];
        $shipsParCarriers = [];
        foreach ($carriers as $carrierKey => $carrier) {
            if (!isset($carrierDetails[$carrier['id_carrier']])) {
                $carrierDetails[$carrier['id_carrier']] = [
                    'y' => [],
                    'n' => [],
                    'm' => []
                ];
            }

            foreach ($products as $productKey => $product) {
                $productCarriers = ($product['id_carriers'] != '') ? explode(',', $product['id_carriers']) : [];
                if (count($productCarriers) == 0 || in_array($carrier['id_carrier'], $productCarriers)) {
                    $carrierDetails[$carrier['id_carrier']]['y'][] = $productKey;
                } else {
                    $carrierDetails[$carrier['id_carrier']]['n'][] = $productKey;
                    $carrierDetails[$carrier['id_carrier']]['p'][$productKey] = $product;
                }
            }

            if (count($carrierDetails[$carrier['id_carrier']]['n']) == 0) {
                $shipsAllCarriers[$carrierKey] = $carrier;
            } else {
                if (count($carrierDetails[$carrier['id_carrier']]['y']) > 0) {
                    $shipsParCarriers[$carrierKey] = $carrier;
                } else {
                    unset($carrierDetails[$carrier['id_carrier']]);
                }
            }
        }

        return $this->getCarriersForProductsReturn($carriers, $carrierDetails, $shipsAllCarriers, $shipsParCarriers);
    }

    private function getCarriersForProductsReturn($carriers, $carrierDetails, $shipsAllCarriers, $shipsParCarriers)
    {
        if (count($shipsAllCarriers) > 0) {
            $returnCarriers = [];
            foreach ($shipsAllCarriers as $shipsAllCarrierKey => $shipsAllCarrier) {
                $returnCarriers[$shipsAllCarrier['id_carrier']] = $carrierDetails[$shipsAllCarrier['id_carrier']];
                $returnCarriers[$shipsAllCarrier['id_carrier']]['y-' . $shipsAllCarrier['id_carrier']] = $returnCarriers[$shipsAllCarrier['id_carrier']]['y'];
                unset($returnCarriers[$shipsAllCarrier['id_carrier']]['y']);
                unset($returnCarriers[$shipsAllCarrier['id_carrier']]['n']);
                unset($returnCarriers[$shipsAllCarrier['id_carrier']]['m']);
                unset($returnCarriers[$shipsAllCarrier['id_carrier']]['p']);
            }
            return $returnCarriers;
        } else {
            $searchCarriers = $carriers;
            foreach ($shipsParCarriers as $shipsParCarrierKey => $shipsParCarrier) {
                unset($searchCarriers[$shipsParCarrierKey]);
                $carrierProducts = $this->getCarriersForProducts($carrierDetails[$shipsParCarrier['id_carrier']]['p'], $searchCarriers);
                unset($carrierDetails[$shipsParCarrier['id_carrier']]['p']);
                foreach ($carrierProducts as $carrierProductKey => $carrierProduct) {
                    $carrierProduct['y-' . $shipsParCarrier['id_carrier']] = $carrierDetails[$shipsParCarrier['id_carrier']]['y'];
                    $carrierDetails[$shipsParCarrier['id_carrier'] . '-' . $carrierProductKey] = $carrierProduct;
                }
                unset($carrierDetails[$shipsParCarrier['id_carrier']]);
            }
            return $carrierDetails;
        }
    }
}
