<?php

namespace Kahanit\TableRateShipping\Model\ResourceModel;

// inheritance
use Magento\Framework\Model\ResourceModel\Db\AbstractDb;

class CarrierTableRateProduct extends AbstractDb
{
    protected function _construct()
    {
        $this->_init('carrier_table_rate_product', 'id_carrier_product');
    }

    public function updateProductSelected($id_product, $id_carrier, $select, $id_shop)
    {
        if ($select) {
            return $this->assignCarrierToProduct($id_product, $id_carrier, $id_shop);
        } else {
            return $this->removeCarrierFromProduct($id_product, $id_carrier, $id_shop);
        }
    }

    public function assignCarrierToProduct($id_product, $id_carrier, $id_shop)
    {
        if (!$this->hasProductCarrier($id_product, $id_carrier, $id_shop)) {
            $connection = $this->getConnection();
            $connection->insert(
                $this->getMainTable(),
                [
                    'id_product' => (int)$id_product,
                    'id_carrier' => (int)$id_carrier,
                    'id_shop' => (int)$id_shop
                ]
            );
        }

        return [
            'status' => 'success',
            'message' => 'Product selected',
            'id_product' => (int)$id_product,
            'id_carrier' => (int)$id_carrier,
            'select' => $this->hasProductCarrier($id_product, $id_carrier, $id_shop)
        ];
    }

    public function removeCarrierFromProduct($id_product, $id_carrier, $id_shop)
    {
        if ($this->hasProductCarrier($id_product, $id_carrier, $id_shop)) {
            $connection = $this->getConnection();
            $connection->delete(
                $this->getMainTable(),
                '   id_product = ' . (int)$id_product . '
                AND id_carrier = ' . (int)$id_carrier . '
                AND id_shop = ' . (int)$id_shop
            );
        }

        return [
            'status' => 'success',
            'message' => 'Product deselected',
            'id_product' => (int)$id_product,
            'id_carrier' => (int)$id_carrier,
            'select' => $this->hasProductCarrier($id_product, $id_carrier, $id_shop)
        ];
    }

    public function hasProductCarrier($id_product, $id_carrier, $id_shop)
    {
        $connection = $this->getConnection();
        $sql = $connection->select();
        $sql->from($this->getMainTable(), ['COUNT(id_carrier_product)']);
        $sql->where(
            '   id_product = ' . (int)$id_product . '
            AND id_carrier = ' . (int)$id_carrier . '
            AND id_shop = ' . (int)$id_shop
        );

        return (bool)$connection->fetchOne($sql);
    }

    public function getProductCarriers($id_product, $id_shop)
    {
        $connection = $this->getConnection();
        $sql = $connection->select();
        $sql->from($this->getMainTable(), ['id_carrier']);
        $sql->where(
            '   id_product = ' . (int)$id_product . '
            AND id_shop = ' . (int)$id_shop
        );

        return $connection->fetchAll($sql);
    }

    public function deleteByCarrier($id_carrier)
    {
        $this->getConnection()->delete($this->getMainTable(), 'id_carrier = ' . (int)$id_carrier);
    }

    public function deleteByProduct($id_product)
    {
        $this->getConnection()->delete($this->getMainTable(), 'id_product = ' . (int)$id_product);
    }
}
