<?php

namespace Kahanit\TableRateShipping\Setup;

// inheritance
use Magento\Framework\Setup\UpgradeDataInterface;
// injection
use Kahanit\TableRateShipping\Model\CarrierTableRateFactory;
// other
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Kahanit\TableRateShipping\Helper\TRSHelper;

class UpgradeData implements UpgradeDataInterface
{
    private $carrierTableRateFactory;

    public function __construct(CarrierTableRateFactory $carrierTableRateFactory)
    {
        $this->carrierTableRateFactory = $carrierTableRateFactory;
    }

    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();
        $connection = $installer->getConnection();

        if (version_compare($context->getVersion(), '2.0.1', '<')) {
            // get data from table by carrier
            $sql = $connection->select();
            $sql->from($installer->getTable('carrier_table_rate'), ['*']);
            $results = $connection->fetchAll($sql);
            $connection->delete($installer->getTable('carrier_table_rate'));
            $records = [];

            foreach ($results as &$result) {
                // strip slashes
                $result = TRSHelper::stripslashes($result);

                // set records
                if (!isset($records[$result['id_shop'] . '-' . $result['id_carrier']])
                    || !is_array($records[$result['id_shop'] . '-' . $result['id_carrier']])
                ) {
                    $records[$result['id_shop'] . '-' . $result['id_carrier']] = [];
                }
                $records[$result['id_shop'] . '-' . $result['id_carrier']][] = $result;
            }

            // save rules by shop and carrier
            foreach ($records as $id_carrier_shop => $crecords) {
                $id_carrier_shop_explode = explode('-', $id_carrier_shop);
                if (isset($id_carrier_shop_explode[0]) && is_numeric($id_carrier_shop_explode[0]) && $id_carrier_shop_explode[0] != 0) {
                    $id_shop = $id_carrier_shop_explode[0];
                } else {
                    $id_shop = null;
                }
                if (isset($id_carrier_shop_explode[1]) && is_numeric($id_carrier_shop_explode[1])) {
                    $this->carrierTableRateFactory->create()->getResource()->saveRules('add', $id_carrier_shop_explode[1], $crecords, null, $id_shop);
                }
            }
        }
    }
}
